// Copyright 2009 Thomas Hoppe
// -----------------------------------------
// This work is licensed under the Creative Commons Attribution License. 
// To view a copy of this license, visit http://creativecommons.org/licenses/by/2.0/ 
// or send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
// You are free:
// - to copy, distribute, display, and perform the work
// - to make derivative works
// - to make commercial use of the work
// Under the following conditions:
// - Attribution. You must give the original author credit.
// - For any reuse or distribution, you must make clear to others the license terms of this work.
// - Any of these conditions can be waived if you get permission from the copyright holder.
// Your fair use and other rights are in no way affected by the above. 

#version 3.6;

#include "colors.inc"  
#include "transforms.inc"

#include "nbase.pov"   
#include "ivy.pov"

StdRadiosity(135, 0.04, 0.08, 1.0)

// ----------------------------------------
                       
//Camera(<-1.5, 1.6, -7.0>, <0.0, 0.5,  0.0>)
Camera(<-1.5, 3.6, -7.0>, <1.0, 0.5,  0.0>)
//Camera(<-0.5, 0.8, -2.0>, <-2.0, 0.5,  0.0>)   
//Camera(<-4.5, 1.8, -3.0>, <-2.0, 0.5,  0.0>)  
//Camera(<-1.5, 0.8, -5.0>, 0)
//Camera(<1.5, 0.8, -3.0>, 0)              
//Camera(<-4.5, 4.8, -7.0>, <0.0, 0.5,  0.0>) 
//Camera(<1.3, -.6, -2>, < 1.5, -.95, -1.5>)

sky_sphere {
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
  }
}

light_source {
  <0, 0, 0>            // light's position (translated below)
  color rgb <2, 1.8, 1.8>  // light's color   
/*
  area_light
  <8, 0, 0> <0, 0, 8> // lights spread out across this distance (x * z)
  4, 4                // total number of lights in grid (4x*4z = 16 lights)
  adaptive 1          // 0,1,2,3...
  jitter              // adds random softening of light
  circular            // make the shape of the light circular
  orient              // orient light  
*/  
  translate <-30, 30, -30>   
  
  //shadowless     
}                       

// ----------------------------------------

// Place N spheres in a 5x5x5 box around 0,
// clamp to given object and place "hairs" at this points.
#macro ClampTest(Seed, Obj, Thick, Len, Steps)
	#local N=10000;																// Count of spheres.
	#local i=0;																	// clamp loop var.
	#while(i<N)																	// clamp loop.
		#local P=vrnd(5, 5, 5, Seed);										// Random position.
				                   
		#local Norm=<0, 0, 0>;													// Normal for clamp detection.		                   
		#local Cl=Clamp(P, Obj, 8, 8, Norm);									// Find nearest intersection.    
		#if(vlength(Norm)!=0)													// Intersection detected ?
			sphere { Cl, 0.01 texture { pigment { color Red } } }				// Place sphere at intersection.
			#local N0=0;														// Append green cylinders in normal dir.                    
			#local Grav=y*-.02;
			#while(N0<Steps)
				#local E=Cl+Norm*Len+Grav;
				cylinder { Cl, E, Thick texture { pigment { color Green } } }	
				#local Cl=E;            
				#local Grav=Grav+Grav;
				#local N0=N0+1;
			#end
		#end
		#local i=i+1;															// Next sphere.
	#end
#end 

#declare Obj2=
union
{
	cylinder { 0, y*2, .8  }
	cone { y*2, .8, y*2.5, .2 }
	sphere { y*3, .5 } 
	
	translate y*-1
	//rotate z*30
	                      
	pigment { color rgbt .7}
}  

#declare Obj=
merge
{   
	difference 
	{    
		merge
		{
			box { <-2, 0, -.5>, <2, 3, .5> } 
			box { <-12, 0, -.25>, <12, 2, .25> }
		}
		box { <-1, -1, -2>, <1, 2, 2> }
	}
	cylinder { <0, 3, 0>, <0, 4, 0>, 1 }
	plane { y, 0}
	
	translate y*-1
	                      
	pigment { color rgb .7}  
}            

object 
{ 
	Obj 
	pigment 
	{
		color rgb .8 
	} 
} 

//ClampTest(Obj, 0.002, 0.05, 5)

//object { IvyLeafObj(seed(123), <0, -1, 0>, <1, 1, 0>, -z, .7, .5) scale 2 rotate y*0 }    
//BaseArrows(1, 0.025, 0, 0)
/*  
#local i=0;    
#local S=seed(1);
#while(i<300)
	#local P=vrnd(5, 5, 5, S);
	#local N=<0, 0, 0>;                                
	#local I=Clamp(P, Obj, 20, N);
	#if(vlength(N)!=0)
		sphere { P, 0.007 pigment { color Red }	 }
		sphere { I, 0.007 pigment { color Red }	 }
		cylinder { I, P, 0.003 pigment { color Blue } }
	#end
	#local i=i+1;
#end
*/                                                                       

// how to use:
// - start with "generate leaves=false", recursion depth=0, set start point/direction near obj surface
//   max segments sould be 100-200 at this moment
// - adjust weight factors, twig thickness/twig segment len, so the main twig gets the right dimensions
// - set recursion depth to 1, use a low "chance for twig split", increase it, maybe try depth=2
//   with low "chance for twig split", adjust weights and length again, remember that every "seed" brings
//   a totally different result
// - set recursion depth to 0, enable "generate leaves", adjust leaf scaling factor
// - reset to previous recursion depth
// - ... experiment with all params :-)
Ivy(
	27,                           // seed base, many streams above will be used
	< 1.5, -.95, -1.5>,            // start point
	<-.3, 0, 1>,                            // start direction
	Obj,                          // object(s) to interact with (may be a placeholder)
	90,                          // max segments per twig
	.08,                          // twig segment len
	.011,                         // twig (starting) thickness (cylinder radius)
	1,                            // max recursion depth (twig splitting), 1 useful, 2 maybe, >2 never tested :)
	.15,                          // chance for twig split (0.05 = 5%)
	.05,                           // minimum distance twigs->object surface
	.02,                          // variance of the above (added)
	.4,                           // weight factor for RANDOM grow direction
	.6,                           // weight factor for PREVIOUS DIR grow direction
	.18,                          // weight factor for UP dir grow direction
	.02,                          // weight factor for GRAVITY/DOWN grow direction     
	.17,                          // leaf scaling factor
	2.0                           // leaves per twig element
	true,                        // generate leaves (false for 'dry run')
	true,                         // progress info, let this on true
	false                          // debug generation, developers friend :-)
	)  
